DatingGameInterpreter = function(g) {
	var that = {};

	that.score = function(g, item) {
		g.score(item.v);
		return false;
	}

	that.goto = function(g, item) {
		console.log(item.v);
		g.places[item.v](g);
		return false;
	}

	that.push = function(g, item) {
		that.save(g, item);
		return that.goto(g, item);
	}

	that.save = function(g, item) {
		g.queue.save();
		return false;
	}

	that.seq = function(g, item) {
		var list = item.v;
		for(var i = 0; i < list.length; i++) {
			g.interpret(list[i]);
		}
		return false;
	}

	that.background = function(g, item) {
		g.view.setBG(item.v);
		return false;
	}

	that.image = function(g, item) {
		g.view.image(item.w, item.v);
		return false;
	}

	that.text = function(g, item) {
		g.view.text(item.w, item.v);
		g.spacePressed = false;
		g.res.sfx.text.bleep();
		if(item.wait == false) {
			return false;
		} else {
			return function() {return g.spacePressed};
		}
	}

	that.space = function(g, item) {
		return function() {return g.spacePressed};		
	}

	that.wait = function(g, item) {
		var time = g.ct + item.v;
		return function() {return g.ct > time};
	}

	that.choice = function(g, item) {
		g.generateChoice(item.v);
		return function() {return g.choiceMade};
	}

	return that;
}

DatingGame = function(canvas, choiceDiv, res) {
	var that = {};

	// http://forums.mozillazine.org/viewtopic.php?f=38&t=722865
	// Disable default space bar behaviour on Firefox
	function dSpace(e){if((e.keyCode==32)&&(!e.shiftKey)){e.preventDefault();}}window.addEventListener('keydown',dSpace,true);
	
	that.res = res;
	that.queue = DatingQueue();
	that.view = DatingView(canvas, res);
	that.interpreter = DatingGameInterpreter(that);
	that.places = DatingGamePlaces;
	that.state = {
		hasVisitedTown: false,
		doctorQuestion: 0,
		points: {
			Jenny: 0,
			Susan: 0,
			Terry: 0
		},
		knownAs: {
			Jenny: "",
			Susan: "",
			Terry: ""
		}
	};

	that.ct = 0;

	that.res.sfx.back.loop = true;

	that.push = function(item) {that.queue.push(item);};
	that.pop = function() {return that.pop();};
	that.clear = function() {return that.clear();};

	that.sequence = function(items) {
		for(var i = 0; i < items.length; i++) {
			that.push(items[i]);
		}
	}

	that.spacePressed = false;
	that.choiceMade = false;

	// this is a hack, i don't get JS closures :-/
	that.choiceDict = {}

	that.score = function(points) {
		that.state.points[that.state.currentGirl.name] += points;
		if(points == 0) {
			// NOOP!
		}
		else if(points < 0) {
			res.sfx.sad.bleep();
		} else {
			res.sfx.happy.bleep();
		}
	}

	that.advanceCondition = function() {return false;};

	that.generateChoice = function(items) {
		that.choiceDict = {};

		for(var i = 0; i < items.length; i++) {
			var item = items[i];
			var span = document.createElement("span");
			var text = document.createTextNode(item[0]);
			span.appendChild(text);
			choiceDiv.appendChild(span);
			
			that.choiceDict[item[0]] = item[1];

			span.onclick = function(e) {
				e.stopPropagation();
				var el = e.toElement;
				if(el == undefined) {
					el = e.originalTarget;
				}
				console.log(el.innerHTML);
				that.choose(that.choiceDict[el.innerHTML]);
			}
		}
	}

	that.choose = function(item) {
		that.interpret(item);
		while(choiceDiv.childNodes.length >= 1) {
			choiceDiv.removeChild(choiceDiv.firstChild);
		}

		that.choiceMade = true;
	}

	that.next = function() {
		var item = that.queue.pop();
		// console.log(item);
		if(item == undefined) {
			console.log("End of Game?");
		} else {
			var ret = that.interpret(item);
			if(ret == false) {
				that.next();
			} else {
				that.advanceCondition = ret;
			}
		}
	}

	that.interpret = function(item) {
		var action = item.a;
		if((typeof action) == "string") {
			action = that.interpreter[item.a];
		}
		console.log(action);
		return action(that, item);
	}

	that.update = function(dt) {
		that.ct = that.ct + dt;

		if(that.introPhase == 2 && that.ct > 3) {
			that.introPhase1();
		}

		if(that.introPhase == 1 && that.spacePressed) {
			that.introPhase0();
		}

		if(that.introPhase == 1 && that.ct > 7) {
			that.introPhase0();
		}

		// intro sollte eigentlich besser eigener state sein.
		if(that.introPhase == 0) {
			// check if we can advance
			if(that.advanceCondition()) {
				that.advanceCondition = function() {return false};
				that.next();
			}

			that.view.update(dt);
		}

		// clear flags
		that.spacePressed = false;
		that.choiceMade = false;
	}

	that.draw = function() {
		ctx.fillStyle = "black";
		ctx.fillRect(0, 0, canvas.width, canvas.height);

		if(that.introPhase > 0) {
			ctx.fillStyle = "white";
			ctx.font = "80px VT323";
			ctx.fillText("Dr. Spackolo", canvas.width/2, canvas.height/2 - 30)
			if(that.introPhase == 1) {
				ctx.font = "40px VT323";
				ctx.fillText("Evil Geniuses need Love too!", canvas.width/2, canvas.height/2 + 30);
			}
		}
		else if(that.introPhase == 0) {
			that.view.draw();
		}
	}

	that.start = function() {
		that.push({a: "goto", v: "start"});
		that.next();
	}

	window.onkeyup = function(e) {
		if(e.keyCode == 32) {
			that.spacePressed = true;
			console.log("Space!");
		}
	}

	that.introPhase2 = function() {
		that.introPhase = 2;
		res.sfx.introa.bleep();

	}

	that.introPhase1 = function() {
		that.introPhase = 1;
		res.sfx.introb.bleep();

	}

	that.introPhase0 = function() {
		that.introPhase = 0;

		ctx.textBaseLine = "alphabetic";
		ctx.textAlign = "start";

		that.start();
		res.sfx.back.play();
	}

	that.introPhase2();

	return that;
}

DatingQueue = function() {
	var that = {};
	that.list = [];
	that.stack = [];

	that.save = function() {
		that.stack.push(that.list);
		that.list = [];
	}

	that.restore = function() {
		that.list = that.stack.pop();
	}

	that.push = function(item) {
		that.list.push(item);
	}

	that.pop = function() {
		if(that.list.length > 0) {
			return that.list.shift();
		} else if(that.stack.length > 0) {
			that.restore();
			return that.pop();
		}
	}

	that.clear = function() {
		that.list = [];
	}

	return that;
}

DatingView = function(canvas, res) {
	var that = {};
	var width = canvas.width;
	var height = canvas.height;

	var ctx = canvas.getContext('2d');

	ctx.webkitImageSmoothingEnabled = false;
	ctx.mozImageSmoothingEnabled = false;


	var ptable = {
		you: {
			pos: [width - 50 - 160, height - 50 - 240, 160, 240],
			textBox: [50 ,height-50-200, 500, 200],
			text: "",
			textPos: 0,
			textList: [],
			lastCharClick: 0,
			image: undefined,
			oldImage: undefined,
			oldAlpha: 0,
			fadeFactor: 500
		},
		she: {
			pos: [50, 50, 160, 240],
			textBox: [250, 50, 500, 200],
			text: "",
			textPos: 0,
			textList: [],
			lastCharClick: 0,
			image: undefined,
			oldImage: undefined,
			oldAlpha: 0,
			fadeFactor: 2
		}
	};

	var bgImage = undefined;
	var oldBgImage = undefined;
	var oldBgAlpha = 0;

	var persons = [ptable.you, ptable.she];

	var lastCharClick = 0;

	that.update = function(dt) {
		for (i in persons) {
			var p = persons[i];
			p.lastCharClick = p.lastCharClick + dt;
			if(p.lastCharClick > 0.025 && p.textPos <= p.text.length) {
				p.lastCharClick -= 0.025;
				p.textPos = Math.min(p.text.length, p.textPos + 1);
				that.buildTextList(p);
			}
			if(p.oldAlpha > 0) {
				p.oldAlpha = Math.max(0, p.oldAlpha - p.fadeFactor * dt);
			}
		}
		if(oldBgAlpha > 0) {
			oldBgAlpha = Math.max(0, oldBgAlpha - 1.5 * dt);
		}
	}

	that.draw = function() {
		if(bgImage != undefined) {
			ctx.globalAlpha = 1 - oldBgAlpha;
			ctx.drawImage(bgImage, 0, 0, width, height);
		}
		if(oldBgImage != undefined && oldBgAlpha > 0.001) {
			ctx.globalAlpha = oldBgAlpha;
			ctx.drawImage(oldBgImage, 0, 0, width, height);
		}

		ctx.globalAlpha = 1;

		for (i in persons) {
			var p = persons[i];

			ctx.globalAlpha = 0.5;
			ctx.fillStyle = "black";
			ctx.fillRect(p.pos[0] , p.pos[1], p.pos[2], p.pos[3]);
			ctx.fillRect(p.textBox[0], p.textBox[1], p.textBox[2], p.textBox[3]);
			ctx.globalAlpha = 1;


			if(p.image != undefined) {
				ctx.globalAlpha = 1 - p.oldAlpha;
				ctx.drawImage(p.image, p.pos[0], p.pos[1], p.pos[2], p.pos[3]);
			}
			if(p.oldImage != undefined && p.oldAlpha > 0.001) {
				ctx.globalAlpha = p.oldAlpha;
				ctx.drawImage(p.oldImage, p.pos[0], p.pos[1], p.pos[2], p.pos[3]);
			}

			ctx.globalAlpha = 1;


			ctx.fillStyle = "white";
			ctx.font = "32px VT323";
			var tpx = p.textBox[0] + 10;
			var tpy = p.textBox[1] + 35;
			for(var i = 0; i < p.textList.length; i++) {
				ctx.fillText(p.textList[i], tpx, tpy);
				tpy += 35;
			}

			ctx.strokeStyle = "white";
			ctx.strokeRect(p.pos[0] , p.pos[1], p.pos[2], p.pos[3]);
			ctx.strokeRect(p.textBox[0], p.textBox[1], p.textBox[2], p.textBox[3]);

		}
	}
	// 36
	that.buildTextList = function(p) {
		var list = [];
		var wholeWords = p.text.split(" ");
		var words = p.text.substr(0, p.textPos).split(" ");
		var line = "";
		for(var i = 0; i < words.length; i++) {
			var word = words[i];
			var wholeWord = wholeWords[i];
			if(line.length + wholeWord.length >= 35) {
				list.push(line);
				line = "";
			}
			line = line + " " + word;
		}
		list.push(line);
		p.textList = list;
	}

	that.setBG = function(newBG) {
		oldBgImage = bgImage;
		bgImage = res.img[newBG];
		oldBgAlpha = 1;
	}

	that.image = function(who, newImage) {
		var p = ptable[who];
		p.oldImage = p.image;
		p.oldAlpha = 1;
		p.image = res.img[newImage];
	}

	that.clear = function(who) {
		that.set(who, undefined);
	}

	that.text = function(who, text) {

		for (i in persons) {
			var pp = persons[i];
			pp.textPos = pp.text.length - 1;
		}

		var p = ptable[who];
		p.textPos = 0;
		p.text = text;
		that.buildTextList(p);
	}

	that.choice = function(choice) {

	}

	that.clearChoice = function() {

	}

	that.cancelAnimations = function() {

	}


	return that;
}